﻿using System.Data;
using System.Data.Common;
using NHibernate.Dialect;
using NHibernate.Engine;
using NHibernate.SqlTypes;
using NHibernate.Type;
using NodaTime;

namespace Drive.Common.Data
{
    public class InstantType : PrimitiveType
    {
        public InstantType()
            : base(SqlTypeFactory.DateTime)
        { }

        public override string Name
        {
            get { return "Instant"; }
        }

        public override Type ReturnedClass
        {
            get { return typeof(Instant); }
        }

        public override object DefaultValue
        {
            get { return default(Instant); }
        }

        public override Type PrimitiveClass
        {
            get { return typeof(Instant); }
        }

        public override object Get(DbDataReader rs, int index, ISessionImplementor session)
        {
            try
            {
                DateTime dateTime = Convert.ToDateTime(rs[index]); // we get an unspecified datetime from the reader, although it really is a UTC datetime
                DateTime utc = new(dateTime.Year, dateTime.Month, dateTime.Day, dateTime.Hour, dateTime.Minute, dateTime.Second, dateTime.Millisecond, DateTimeKind.Utc); // force the same datetime value to be a UTC value
                return Instant.FromDateTimeUtc(utc);
            }
            catch (Exception ex)
            {
                throw new FormatException(string.Format("Input string '{0}' was not in the correct format.", rs[index]), ex);
            }
        }

        public override object Get(DbDataReader rs, string name, ISessionImplementor session)
        {
            return Get(rs, rs.GetOrdinal(name), session);
        }

        public override void Set(DbCommand cmd, object value, int index, ISessionImplementor session)
        {
            var parameter = (IDataParameter)cmd.Parameters[index];
            if (value == null)
                parameter.Value = DBNull.Value;
            else
            {
                DateTime utc = ((Instant)value).ToDateTimeUtc();
                DateTime unspecified = new(utc.Year, utc.Month, utc.Day, utc.Hour, utc.Minute, utc.Second, utc.Millisecond, DateTimeKind.Unspecified);
                parameter.DbType = DbType.DateTime;
                parameter.Value = unspecified;
            }
        }

        public override string ObjectToSQLString(object value, Dialect dialect)
        {
            return ((Instant)value).ToDateTimeUtc().ToString("'yyyyMMdd HH:mm:ss'", null);
        }
    }
}
