﻿using System.Data.Common;
using NHibernate;
using NHibernate.Engine;
using NHibernate.SqlTypes;
using NHibernate.UserTypes;
using NodaTime;

namespace Drive.Common.Data
{
    public class InstantVersionType : IUserVersionType
    {
        public SqlType[] SqlTypes => new[] { SqlTypeFactory.DateTime2 };

        public Type ReturnedType => typeof(Instant);

        public bool IsMutable => false;

        public object Assemble(object cached, object owner)
        {
            return DeepCopy(cached);
        }

        public object DeepCopy(object value)
        {
            return value;
        }

        public object Disassemble(object value)
        {
            return DeepCopy(value);
        }

        public new bool Equals(object x, object y)
        {
            return x.Equals(y);
        }

        public int GetHashCode(object x)
        {
            return x.GetHashCode();
        }

        public object NullSafeGet(DbDataReader rs, string[] names, ISessionImplementor session, object owner)
        {
            DateTime dateTime = Convert.ToDateTime(rs[names[0]]);
            DateTime utc = new(
                dateTime.Ticks,
                DateTimeKind.Utc);
            return Instant.FromDateTimeUtc(utc);
        }

        public void NullSafeSet(DbCommand cmd, object value, int index, ISessionImplementor session)
        {
            DateTime utc = ((Instant)value).ToDateTimeUtc();
            DateTime unspecified = new(
               utc.Ticks,
               DateTimeKind.Unspecified);
            NHibernateUtil.DateTime.NullSafeSet(cmd, unspecified, index, session);
        }

        public object Replace(object original, object target, object owner)
        {
            return original;
        }

        public int Compare(object? x, object? y)
        {
            if (x is Instant xx && y is Instant yy)
            {
                return xx.CompareTo(yy);
            }

            return -1;
        }

        public object Next(object current, ISessionImplementor session)
        {
            return Seed(session);
        }

        public object Seed(ISessionImplementor session)
        {
            return Instant.FromDateTimeUtc(DateTime.UtcNow);
        }
    }
}
