﻿using System.Data;
using System.Data.Common;
using System.Text.Json;
using NHibernate.Engine;
using NHibernate.SqlTypes;
using NHibernate.Type;

namespace Drive.Common.Data
{
    public class JsonType<T> :
        MutableType
    {
        public JsonType() : base(new StringClobSqlType())
        {
        }

        public override string Name => nameof(T);

        public override Type ReturnedClass => typeof(T);

        public override object? Get(DbDataReader rs, int index, ISessionImplementor session)
        {
            try
            {
                T? obj = JsonSerializer.Deserialize<T>(rs[index]?.ToString() ?? string.Empty, DriveSerialization.Options);
                return obj;
            }
            catch (Exception ex)
            {
                throw new FormatException(string.Format("Input string '{0}' was not in the correct format.", rs[index]), ex);
            }
            throw new NotImplementedException();
        }

        public override object? Get(DbDataReader rs, string name, ISessionImplementor session)
        {
            return Get(rs, rs.GetOrdinal(name), session);
        }

        public override void Set(DbCommand cmd, object value, int index, ISessionImplementor session)
        {
            var parm = cmd.Parameters[index] as IDataParameter;
            if (value == null)
                parm.Value = DBNull.Value;
            else
            {
                parm.DbType = DbType.String;
                parm.Value = JsonSerializer.Serialize(value, DriveSerialization.Options);
            }
        }

        public override object? DeepCopyNotNull(object value)
        {
            string json = JsonSerializer.Serialize(value, DriveSerialization.Options);
            return JsonSerializer.Deserialize<T>(json, DriveSerialization.Options);
        }
    }
}
