﻿using System.Data;
using System.Data.Common;
using NHibernate.Dialect;
using NHibernate.Engine;
using NHibernate.SqlTypes;
using NHibernate.Type;
using NodaTime;

namespace Drive.Common.Data
{
    public class LocalDateType : PrimitiveType
    {
        public LocalDateType()
            : base(SqlTypeFactory.Date)
        {
        }

        public override string Name
        {
            get { return "LocalDate"; }
        }

        public override Type ReturnedClass
        {
            get { return typeof(LocalDate); }
        }

        public override object DefaultValue
        {
            get { return default(LocalDate); }
        }

        public override Type PrimitiveClass
        {
            get { return typeof(LocalDate); }
        }

        public override object Get(DbDataReader rs, int index, ISessionImplementor session)
        {
            try
            {
                var dateTime = Convert.ToDateTime(rs[index]);
                return LocalDateTime.FromDateTime(dateTime).Date;
            }
            catch (Exception ex)
            {
                throw new FormatException(string.Format("Input string '{0}' was not in the correct format.", rs[index]), ex);
            }
        }

        public override object Get(DbDataReader rs, string name, ISessionImplementor session)
        {
            return Get(rs, rs.GetOrdinal(name), session);
        }

        public override void Set(DbCommand cmd, object value, int index, ISessionImplementor session)
        {
            var parm = cmd.Parameters[index] as IDataParameter;
            if (value == null)
                parm.Value = DBNull.Value;
            else
            {
                var localDate = (LocalDate)value;
                parm.DbType = DbType.Date;
                parm.Value = localDate.AtMidnight().ToDateTimeUnspecified();
            }
        }

        public override string ObjectToSQLString(object value, Dialect dialect)
        {
            return ((LocalDate)value).ToString("'yyyyMMdd'", null);
        }

        public override bool IsEqual(object x, object y)
        {
            if (x == y)
                return true;
            if (x == null || y == null)
                return false;

            return ((LocalDate)x).Equals(y);
        }

        public override int GetHashCode(object x, ISessionFactoryImplementor factory)
        {
            var date = (LocalDate)x;
            var hashCode = 1;
            unchecked
            {
                hashCode = 31 * hashCode + date.Day;
                hashCode = 31 * hashCode + date.Month;
                hashCode = 31 * hashCode + date.Year;
            }
            return hashCode;
        }

        public override string? ToLoggableString(object val, ISessionFactoryImplementor factory)
        {
            return val is not null && val is LocalDate actual ? actual.ToString("'yyyyMMdd'", null) : null;
        }
    }
}