﻿using System.Data.Common;
using Drive.Common.Data.DialectExtensions;
using Microsoft.Extensions.Options;
using NHibernate;
using NHibernate.Cfg;
using NHibernate.Driver;

namespace Drive.Common.Data
{
    public class NHibernateSessionSource :
        ISessionSource
    {
        private readonly OrmOptions _options;
        private readonly ISessionFactory _sessionFactory;
        private readonly Configuration _configuration;

        public NHibernateSessionSource(
            IOptions<OrmOptions> options,
            IConfigureSessionFactory[] configurers)
        {
            _options = options.Value;
            _configuration = AssembleConfiguration(configurers);
            _sessionFactory = _configuration.BuildSessionFactory();
        }

        public Configuration Configuration { get { return _configuration; } }

        public ISession CreateSession(
            DbConnection connection,
            IInterceptor? interceptor = null)
        {
            ISessionBuilder builder = _sessionFactory
                .WithOptions()
                .Connection(connection);
            if (interceptor != null)
                builder = builder.Interceptor(interceptor);

            return builder.OpenSession();
        }

        public IStatelessSession CreateStatelessSession(
            DbConnection connection)
        {
            return _sessionFactory
                .OpenStatelessSession(connection);
        }

        Configuration AssembleConfiguration(IEnumerable<IConfigureSessionFactory> configurers)
        {
            Configuration configuration = new();

            configuration.DataBaseIntegration(c =>
            {
                c.Driver<MicrosoftDataSqlClientDriver>();
                c.Dialect<DriveMsSql2012Dialect>();
                c.ConnectionString = _options.ConnectionString;
                c.BatchSize = _options.BatchSize ?? 20;
            });
            configuration.SetProperty(NHibernate.Cfg.Environment.UseConnectionOnSystemTransactionPrepare, "false");
            configuration.LinqToHqlGeneratorsRegistry<ExtendedLinqToHqlGeneratorsRegistry>();

            foreach (var configurer in configurers)
                configuration = configurer.Configure(configuration);

            return configuration;
        }
    }
}
