﻿using System.Security.Claims;
using Drive.Common.Domain;
using Drive.Common.Security;
using Drive.Common.Security.Extensions;

namespace Drive.Common.Messaging
{
    public static class MessageHandlerContextExtensions
    {
        public static async Task PublishAggregateEvents<TRecord>(
            this IMessageHandlerContext context,
            IAggregateRoot<TRecord> aggregate)
            where TRecord : IAggregateRootRecord
        {
            var events = aggregate.GetEvents();
            foreach (var e in events)
            {
                await context
                    .Publish(e)
                    .ConfigureAwait(false);
            }
        }

        public static async Task PublishAggregateEvents<TRecord>(
            this IMessageHandlerContext context,
            ILegacyAggregateRoot<TRecord> aggregate)
            where TRecord : ILegacyAggregateRootRecord
        {
            var events = aggregate.GetEvents();
            foreach (var e in events)
            {
                await context
                    .Publish(e)
                    .ConfigureAwait(false);
            }
        }

        public static DriveContext ToDriveContext(this IMessageHandlerContext context)
        {
            DriveContext ourContext = new()
            {
                DriveUserId = context.Extensions.Get<ClaimsPrincipal>().DriveUserId()
            };

            return ourContext;
        }
    }
}
