﻿using System.Security.Claims;
using Dapper;
using Drive.Common.Security;
using Microsoft.Data.SqlClient;
using Microsoft.IdentityModel.Tokens;
using NServiceBus.Extensibility;

namespace Drive.Common.Messaging.Signing
{
    public static class AdvertisingManagerClaimsPrincipalSendOptionsFactory
    {
        public static SendOptions CreateSendOptions(string connectionString)
        {
            SendOptions sendOptions = new();
            sendOptions.RouteToThisEndpoint();

            using SqlConnection connection = new(connectionString);
            connection.Open();

            User user = connection.QuerySingle<User>(@"
SELECT  TOP 1
        HelperUser.Code AS UserCode,
        HelperUser.OID AS UserId,
        Staff.OID AS StaffId,
        Staff.Branch AS UserBranchId
FROM    AdvertisingManager
INNER JOIN [User] HelperUser ON HelperUser.OID = AdvertisingManager.HelperUser
INNER JOIN Staff ON Staff.OID = HelperUser.Staff;
");

            List<Claim> claims = new()
            {
                new Claim(ClaimTypes.Name, user.UserCode),
                new Claim(DriveClaimTypes.UserId, user.UserId.ToString()),
                new Claim(DriveClaimTypes.StaffId, user.StaffId.ToString()),
                new Claim(DriveClaimTypes.UserBranchId, user.UserBranchId.ToString())
            };
            ClaimsIdentity identity = new(claims, TokenValidationParameters.DefaultAuthenticationType);
            sendOptions.GetExtensions().Set(new ClaimsPrincipal(identity));

            return sendOptions;
        }

        private class User
        {
            public required string UserCode { get; set; }
            public int UserId { get; set; }
            public int StaffId { get; set; }
            public int UserBranchId { get; set; }
        }
    }
}