﻿using System.Security.Claims;
using Drive.Common.Security;
using NServiceBus.Pipeline;

namespace Drive.Common.Messaging.Signing
{
    public class SignatureVerificationBehavior
        : Behavior<IIncomingPhysicalMessageContext>
    {
        private readonly MessageHasher _hasher;
        private readonly SigningTokenValidator _validator;

        public SignatureVerificationBehavior(
            MessageHasher hasher,
            SigningTokenValidator validator)
        {
            _hasher = hasher;
            _validator = validator;
        }

        public override Task Invoke(IIncomingPhysicalMessageContext context, Func<Task> next)
        {
            if (context.MessageHeaders.TryGetValue(HeaderConstants.SIGNING_TOKEN, out string? token))
            {
                ClaimsPrincipal principal = _validator.Validate(token);
                string signature = principal.FindFirst(DriveClaimTypes.MessageHash)!.Value;
                string hash = _hasher.Hash(context.Message.Body);

                if (hash == signature)
                {
                    context.Extensions.Set<ClaimsPrincipal>(principal);
                    return next();
                }
            }
            else if (context.MessageHeaders.TryGetValue(Headers.MessageIntent, out string? rawIntent))
            {
                MessageIntent intent = (MessageIntent)Enum.Parse(typeof(MessageIntent), rawIntent, true);
                if (intent == MessageIntent.Subscribe || intent == MessageIntent.Unsubscribe)
                    return next();
            }

            throw new MessageSignatureException();
        }
    }
}
