﻿using System.Security.Claims;

namespace Drive.Common.Security
{
    public static class ClaimsPrincipalExtensions
    {
        public static bool HasClaim(this ClaimsPrincipal principal, string role)
        {
            return principal.HasClaim(ClaimTypes.Role, role);
        }

        public static bool IsAccountManager(this ClaimsPrincipal principal)
        {
            return principal.HasClaim(RolesManager.DriveAccountManager);
        }

        public static int DriveUserId(this ClaimsPrincipal principal)
        {
            bool authenticated = principal.Identity is not null && principal.Identity.IsAuthenticated;
            if (!authenticated) throw new InvalidOperationException($"{nameof(DriveUserId)} requires authentication");

            Claim claim = principal.FindFirst(DriveClaimTypes.UserId)!;
            return int.Parse(claim.Value);
        }

        public static int DriveUserBranchId(this ClaimsPrincipal principal)
        {
            bool authenticated = principal.Identity is not null && principal.Identity.IsAuthenticated;
            if (!authenticated) throw new InvalidOperationException($"{nameof(DriveUserBranchId)} requires authentication");

            Claim claim = principal.FindFirst(DriveClaimTypes.UserBranchId)!;
            return int.Parse(claim.Value);
        }

        public static int DriveStaffId(this ClaimsPrincipal principal)
        {
            bool authenticated = principal.Identity is not null && principal.Identity.IsAuthenticated;
            if (!authenticated) throw new InvalidOperationException($"{nameof(DriveStaffId)} requires authentication");

            Claim claim = principal.FindFirst(DriveClaimTypes.StaffId)!;
            return int.Parse(claim.Value);
        }

        public static string DriveUserCode(this ClaimsPrincipal principal)
        {
            bool authenticated = principal.Identity is not null && principal.Identity.IsAuthenticated;
            if (!authenticated) throw new InvalidOperationException($"{nameof(DriveUserCode)} requires authentication");

            Claim claim = principal.FindFirst(ClaimTypes.Name)!;
            return claim.Value;
        }

        public static string DriveUserDescription(this ClaimsPrincipal principal)
        {
            bool authenticated = principal.Identity is not null && principal.Identity.IsAuthenticated;
            if (!authenticated) throw new InvalidOperationException($"{nameof(DriveUserDescription)} requires authentication");

            Claim claim = principal.FindFirst(DriveClaimTypes.UserDescription)!;
            return claim.Value;
        }
    }
}
