﻿using System.IdentityModel.Tokens.Jwt;
using System.Security.Claims;
using Microsoft.AspNetCore.Authentication.JwtBearer;
using Microsoft.Extensions.Options;
using Microsoft.IdentityModel.Tokens;

namespace Drive.Common.Security.Token
{
    public class JwtTokenValidator :
        ITokenValidator
    {
        private readonly IOptionsMonitor<JwtBearerOptions> _jwtOptions;
        private readonly ISecurityKeyResolver _resolver;

        public JwtTokenValidator(
            IOptionsMonitor<JwtBearerOptions> jwtOptions,
            ISecurityKeyResolver resolver)
        {
            _jwtOptions = jwtOptions;
            _resolver = resolver;
        }

        public ClaimsPrincipal ValidateToken(string token, TimeSpan? clockSkew = null)
        {
            JwtBearerOptions options = _jwtOptions.Get(JwtBearerDefaults.AuthenticationScheme);

            JwtSecurityTokenHandler tokenHandler = new();
            TokenValidationParameters validationParameters = new()
            {
                ValidAudiences = options.TokenValidationParameters.ValidAudiences,
                ValidAudience = options.TokenValidationParameters.ValidAudience,
                ValidIssuers = options.TokenValidationParameters.ValidIssuers,
                ValidIssuer = options.TokenValidationParameters.ValidIssuer,

                IssuerSigningKeyResolver = (string rawToken, SecurityToken securityToken, string keyId, TokenValidationParameters param) =>
                {
                    return _resolver.GetSecurityKeys(keyId);
                },
                ClockSkew = clockSkew ?? TimeSpan.FromMinutes(5),
            };

            return tokenHandler.ValidateToken(token, validationParameters, out _);
        }
    }
}