﻿using System.Security.Claims;
using Drive.Common.Security;

namespace Drive.Common.Testing.Security
{
    public abstract class AuthenticationState
    {
        public const string USER_CODE = "TEST.USER.CODE";

        public const int AUTHENTICATED_USER_ID = 12345;
        public const int USER_STAFF_ID = 6789;
        public const int STAFF_BRANCH_ID = 47;

        public ClaimsPrincipal CreatePrincipal()
        {
            List<Claim> claims = new()
            {
                new Claim(ClaimTypes.Name, USER_CODE),
                new Claim(DriveClaimTypes.UserId, AUTHENTICATED_USER_ID.ToString()),
                new Claim(DriveClaimTypes.StaffId, USER_STAFF_ID.ToString()),
                new Claim(DriveClaimTypes.UserBranchId, STAFF_BRANCH_ID.ToString()),
            };
            claims.AddRange(GetClaims());

            ClaimsIdentity identity = new(claims, "TEST.Authentication");
            ClaimsPrincipal accountManager = new(identity);

            return accountManager;
        }

        public abstract IEnumerable<Claim> GetClaims();
    }

    public class AccountManager : AuthenticationState
    {
        public override IEnumerable<Claim> GetClaims()
        {
            return new[] { new Claim(ClaimTypes.Role, RolesManager.DriveAccountManager) };
        }
    }
}