﻿using FluentFTP;

namespace Drive.Common
{
    public static class Ftp
    {
        public static Task MakeDirectoryAsync(string directoryName, string ftpSite, string username, string password)
        {
            using FtpClient client = new(ftpSite, username, password);
            client.Connect();
            client.CreateDirectory(directoryName);
            client.Disconnect();

            return Task.CompletedTask;
        }

        public static Task PutAsync(byte[] content, string fileName, string ftpSite, string username, string password)
        {
            using FtpClient client = new(ftpSite, username, password);
            client.Connect();
            client.UploadBytes(content, fileName);
            client.Disconnect();

            return Task.CompletedTask;
        }

        public static Task<byte[]> GetAsync(string fileName, string ftpSite, string username, string password)
        {
            using FtpClient client = new(ftpSite, username, password);
            client.Connect();
            client.DownloadBytes(out byte[] content, fileName);
            client.Disconnect();

            return Task.FromResult(content);
        }

        public static Task DeleteAsync(string fileName, string ftpSite, string username, string password)
        {
            using FtpClient client = new(ftpSite, username, password);
            client.Connect();
            client.DeleteFile(fileName);
            client.Disconnect();

            return Task.CompletedTask;
        }

        public static Task RemoveDirectoryAsync(string directoryName, string ftpSite, string username, string password)
        {
            using FtpClient client = new(ftpSite, username, password);
            client.Connect();
            client.DeleteDirectory(directoryName);
            client.Disconnect();

            return Task.CompletedTask;
        }
    }
}
