﻿using Renci.SshNet;
using Renci.SshNet.Common;
using Renci.SshNet.Sftp;

namespace Drive.Common
{
    [Obsolete("Prefer Renci.SshNet.SftpClient extension methods")]
    public class Sftp : IDisposable
    {
        private readonly SftpClient _client;

        public Sftp(
            string host,
            string userName,
            string password,
            string? authMethod = null
        )
        {
            var connectionInfo = new ConnectionInfo(
                host,
                userName,
                new PasswordAuthenticationMethod(userName, password),
                new PrivateKeyAuthenticationMethod(authMethod ?? "rsa.key"));

            _client = new SftpClient(connectionInfo);
            _client.Connect();
        }

        public void CreateDirectory(string directoryName)
        {
            string[] directories = directoryName.Split(new[] { "/" }, StringSplitOptions.RemoveEmptyEntries);
            string parent = string.Empty;
            foreach (string directory in directories)
            {
                parent = string.Format("{0}/{1}", parent, directory);

                if (DirectoryExists(parent))
                    continue;

                _client.CreateDirectory(parent);
            }
        }

        public IEnumerable<string> ListDirectory(string directory)
        {
            IEnumerable<ISftpFile> directoryContent = _client.ListDirectory(directory);
            IEnumerable<string> result = directoryContent.Select(file => file.FullName).ToList();

            return result;
        }

        public void Upload(byte[] content, string fileName)
        {
            using MemoryStream stream = new(content);

            IAsyncResult asyncResult = _client.BeginUploadFile(stream, fileName);

            asyncResult.AsyncWaitHandle.WaitOne();
        }

        public Task UploadAsync(byte[] content, string fileName)
        {
            Stream stream = Stream.Synchronized(new MemoryStream(content));

            TaskCompletionSource tcs = new();
            _client.BeginUploadFile(stream, fileName, asyncResult =>
            {
                try
                {
                    _client.EndUploadFile(asyncResult);
                    stream.Dispose();

                    tcs.TrySetResult();
                }
                catch (OperationCanceledException)
                {
                    tcs.TrySetCanceled();
                }
                catch (Exception ex)
                {
                    tcs.TrySetException(ex);
                }
            });

            return tcs.Task;
        }

        public byte[] Download(string fileName)
        {
            using MemoryStream stream = new();

            IAsyncResult asyncResult = _client.BeginDownloadFile(fileName, stream);

            asyncResult.AsyncWaitHandle.WaitOne();
            return stream.ToArray();
        }

        public void Delete(string path)
        {
            _client.Delete(path);
        }

        public bool DirectoryExists(string directoryName)
        {
            bool found = true;

            try
            {
                SftpFileAttributes attributes = _client.GetAttributes(directoryName);

                if (!attributes.IsDirectory)
                {
                    throw new ArgumentException($"{directoryName} exists but it's not a directory");
                }
            }
            catch (SftpPathNotFoundException)
            {
                found = false;
            }
            return found;
        }

        public void Dispose()
        {
            if (_client.IsConnected)
            {
                _client.Disconnect();

            }

            _client.Dispose();
        }
    }

    public static class SftpClientExtensions
    {
        public static Task UploadAsync(this SftpClient sftp, byte[] content, string fileName)
        {
            Stream stream = Stream.Synchronized(new MemoryStream(content));

            TaskCompletionSource tcs = new();
            sftp.BeginUploadFile(stream, fileName, asyncResult =>
            {
                try
                {
                    sftp.EndUploadFile(asyncResult);
                    stream.Dispose();

                    tcs.TrySetResult();
                }
                catch (OperationCanceledException)
                {
                    tcs.TrySetCanceled();
                }
                catch (Exception ex)
                {
                    tcs.TrySetException(ex);
                }
            });

            return tcs.Task;
        }

        public static void CreateDirectoryHierarchy(this SftpClient @this, string directoryHierarcy)
        {
            string[] directories = directoryHierarcy.Split(new[] { "/" }, StringSplitOptions.RemoveEmptyEntries);
            string parent = string.Empty;
            foreach (string directory in directories)
            {
                parent = string.Format("{0}/{1}", parent, directory);

                if (@this.DirectoryExists(parent))
                    continue;

                @this.CreateDirectory(parent);
            }
        }

        public static bool DirectoryExists(this SftpClient @this, string directoryName)
        {
            bool found = true;

            try
            {
                SftpFileAttributes attributes = @this.GetAttributes(directoryName);

                if (!attributes.IsDirectory)
                {
                    throw new ArgumentException($"{directoryName} exists but it's not a directory");
                }
            }
            catch (SftpPathNotFoundException)
            {
                found = false;
            }
            return found;
        }
    }
}