﻿using Drive.Common.IntegrationTest.IntegrationDatabaseTest;
using FluentAssertions;
using Xunit;

namespace Drive.Common.IntegrationTest
{
    [CollectionDefinition(nameof(TestIntegrationDatabaseTestCollection))]
    public class TestIntegrationDatabaseTestCollection : ICollectionFixture<TestIntegrationDatabaseTest>
    {
        public const string COLLECTION_NAME = nameof(TestIntegrationDatabaseTestCollection);
    }

    [Collection(TestIntegrationDatabaseTestCollection.COLLECTION_NAME)]
    public class IntegrationDatabaseTestSuite
    {
        private TestIntegrationDatabaseTest _databaseTest;

        public IntegrationDatabaseTestSuite(TestIntegrationDatabaseTest databaseTest)
        {
            _databaseTest = databaseTest;
        }

        [Fact]
        public void can_read_existing_data()
        {
            //arrange
            TestModel test;

            //act
            using (var session = _databaseTest.OpenSession())
            {
                test = session.Query<TestModel>()
                    .Where(x => x.Active && x.Code == _databaseTest.TestCode)
                    .Single();
            }

            //assert
            test.Should().NotBeNull();
            test.Code.Should().Be(_databaseTest.TestCode);
            test.Active.Should().BeTrue();
        }

        [Fact]
        public void can_write_new_data()
        {
            //arrange
            var test = new TestModel()
            {
                Active = true,
                Code = "NEWTEST",
                Description = "New Test Object"
            };

            //act
            using (var session = _databaseTest.OpenSession())
            using (var transaction = session.BeginTransaction())
            {
                session.Save(test);
                transaction.Commit();
            }

            //assert
            test.Id.Should().NotBe(0);
        }

        [Fact]
        public void can_read_existing_data_from_other_schema()
        {
            //arrange
            OtherSchemaModel test;

            //act
            using (var session = _databaseTest.OpenSession())
            {
                test = session.Query<OtherSchemaModel>()
                    .Where(x => x.Active && x.Code == _databaseTest.TestCode)
                    .Single();
            }

            //assert
            test.Should().NotBeNull();
            test.Code.Should().Be(_databaseTest.TestCode);
            test.Active.Should().BeTrue();
        }

        [Fact]
        public void can_write_new_data_to_other_schema()
        {
            //arrange
            var test = new OtherSchemaModel()
            {
                Active = true,
                Code = "OTHERTEST",
                Description = "Other New Test Object"
            };

            //act
            using (var session = _databaseTest.OpenSession())
            using (var transaction = session.BeginTransaction())
            {
                session.Save(test);
                transaction.Commit();
            }

            //assert
            test.Id.Should().NotBe(0);
        }
    }
}