﻿using System.Net;
using Drive.Common.Web.Filter;
using FluentAssertions;
using Microsoft.AspNetCore.Mvc;
using Microsoft.AspNetCore.Mvc.Filters;
using Moq;
using Xunit;

namespace Drive.Common.Web.UnitTest.Filter
{
    public class CorrelationHeadersRequiredAttributeSuite
    {
        [Fact(Skip = "dotnet upgrade, can probably be removed")]
        public void returns_bad_request_when_correlation_id_is_not_set()
        {
            CorrelationHeadersRequiredAttribute attribute = new();
            ActionExecutingContext context = CreateActionContext();

            attribute.OnActionExecuting(context);

            context.Result.Should().NotBeNull();
            context.Result.Should().BeOfType<StatusCodeResult>();
            ((StatusCodeResult)context.Result!).StatusCode.Should().Be((int)HttpStatusCode.BadRequest);
        }

        [Fact(Skip = "dotnet upgrade, can probably be removed")]
        public void response_is_not_set_when_expected_headers_are_set()
        {
            CorrelationHeadersRequiredAttribute attribute = new();
            ActionExecutingContext context = CreateActionContext();

            attribute.OnActionExecuting(context);

            context.Result.Should().BeOfType<StatusCodeResult>();
        }

        public static ActionExecutingContext CreateActionContext()
        {
            ActionContext context = Mock.Of<ActionContext>();
            return new ActionExecutingContext(context, Array.Empty<IFilterMetadata>(), new Dictionary<string, object?>(), new TestController());
        }
    }

    internal class TestController : ControllerBase { }
}
